module Coreimpact
  class Vulnerability
    def initialize(xml_vulnerability)
      @xml = xml_vulnerability
    end

    def supported_tags
      [:agent_deployed, :cve, :description, :port, :tried_to_install_agent, :title]
    end

    def respond_to?(method, include_private = false)
      return true if supported_tags.include?(method.to_sym)

      super
    end

    # Example XML:
    # <property type="container" key="Vulnerabilities">
    #   <property type="container" key="CVE-1999-0516">
    #     <property type="container" key="Modules">
    #       <property type="container" key="<Title>">
    #         <property type="string" key="description">
    #           Some description.
    #         </property>
    #       </property>
    #     </property>
    #   </property>
    # </property>
    def method_missing(method, *args)
      # We could remove this check and return nil for any non-recognized tag.
      # The problem would be that it would make tricky to debug problems with
      # typos. For instance: <>.potr would return nil instead of raising an
      # exception
      unless supported_tags.include?(method)
        super
        return
      end

      result =
        case method
        when :cve
          @xml.first_element_child['key']
        when :title
          @xml.at_xpath('./property/property[@key="Modules"]/property[@type="container"]')['key']
        else
          @xml.at_xpath("./property/property[@key='Modules']/property[@type='container']/property[@key='#{method}']").text
        end

      result || 'n/a'
    end
  end
end
